# -*- coding: utf-8 -*-
"""
@author: Jean-Bernard Hayet
"""

# Importar librerías
import sys
sys.path.insert(0, "/home/jbhayet/opt/NAO/V-REP_PRO_EDU_V3_6_2_Ubuntu18_04/programming/remoteApiBindings/python/python")
from vrep import *         # V-rep library
import time                # Used to keep track of time
import numpy as np         # Array library
import math
import matplotlib.pyplot as plt
import matplotlib.animation as animation

# Valor máximo del sensor de distancia
dmax= 1.0

# Valor de pi
PI=math.pi

# Lista de cpnectores a los sensores
sensor_h  =[]
# Orientación de los sensores:
sensor_loc=np.array([-PI/2, -50/180.0*PI,-30/180.0*PI,-10/180.0*PI,10/180.0*PI,30/180.0*PI,50/180.0*PI,PI/2,PI/2,130/180.0*PI,150/180.0*PI,170/180.0*PI,-170/180.0*PI,-150/180.0*PI,-130/180.0*PI,-PI/2])
front_sensors_ides = range(0,8)

# Esta función dibuja (una vez) los valores de los sensores, en coordenadas polares
def drawSensorValues(sensor_val,ax):
    # Numero de valores que dinujar
    N     = len(sensor_val)
    width = 1.5*PI/ N
    # Aquí se dibujan las barras en polares
    bars  = ax.bar(-sensor_loc, sensor_val, width=width, bottom=0.0)
    fig.canvas.draw()
    fig.canvas.flush_events()
    return bars

# Esta función actualiza el dibujo con nuevos valores
def updateDrawing(sensor_val,ax,bars):
    # Compute pie slices
    for bar, d in zip(bars, sensor_val):
        bar.set_height(d)
    fig.canvas.draw()
    fig.canvas.flush_events()

# Esta función recupera los valores de los sensores
def getSensorValues(clientID):
    sensor_val=np.array([])
    # Iteramos sobre los 16 sensores
    for x in range(1,16+1):
        # Recuperamos el punto de detectado
        error,detection,point,__,__=simxReadProximitySensor(clientID,sensor_h[x-1],simx_opmode_blocking)
        # Nos interesa la distancia entre el robot y este punto
        d = np.linalg.norm(point)
        if  detection:
            sensor_val=np.append(sensor_val,d)
        else:
            # Cuando no hubo detección, ponemos un valor alto.
            sensor_val=np.append(sensor_val,dmax)
    return sensor_val

####### AQUÍ EMPIEZA TODO!

# Eso es para cerrar conexiones pasadas
simxFinish(-1)
# Eso se conecta al robot, con una dirección IP dada.
clientID = simxStart('127.0.0.1',19997,True,True,5000,5)
# Verificamos si estamos bien conectados
if clientID!=-1:
    print('[*] Connected to remote API server')
else:
    print('[!] Connection not successful')
    sys.exit('Could not connect')

# Eso crea la figura en la cual dibujaremos los valores de los sensores
plt.ion()
fig = plt.figure()
ax  = plt.subplot(111, projection='polar')
ax.set_theta_zero_location("N")

# Recuperamos una conexión al motor de izquierda
__,left_motor_handle=simxGetObjectHandle(clientID,'Pioneer_p3dx_leftMotor',simx_opmode_oneshot_wait)
# Recuperamos una conexión al motor de derecha
__,right_motor_handle=simxGetObjectHandle(clientID,'Pioneer_p3dx_rightMotor',simx_opmode_oneshot_wait)

# Recuperamos conexiones a los sensores
for x in range(1,16+1):
    # Recuperamos la conexión al sensor 'x' (un numero entre 1 y 16)
    # y lo ponemos en una lista sensor_h
    errorCode,sensor_handle = simxGetObjectHandle(clientID,'Pioneer_p3dx_ultrasonicSensor'+str(x)+'#0',simx_opmode_blocking)
    sensor_h.append(sensor_handle)

# Al inicio, recuperamos los valores de los sensores y les desplegamos
sensor_val = getSensorValues(clientID)
bars=drawSensorValues(sensor_val,ax)


t = time.time()

# Este es el ciclo durante el cual se produce el experimento
while (time.time()-t)<120:
    print('[*] Tiempo: ',time.time())
    # En cada momento, leemos los sensores y redibujamos la gráfica
    sensor_val = getSensorValues(clientID)
    updateDrawing(sensor_val,ax,bars)

    # Check where the min distance value is reached
    min_ind=np.where(sensor_val[0:8]==np.min(sensor_val[0:8]))
    min_ind=min_ind[0][0]

    vl=0.0
    vr=0.0

    # Pedimos a traves de nuestras conexiones que se ejecute estas
    # velocidades
    errorCode=simxSetJointTargetVelocity(clientID,left_motor_handle,vl, simx_opmode_streaming)
    errorCode=simxSetJointTargetVelocity(clientID,right_motor_handle,vr, simx_opmode_streaming)


    time.sleep(0.2) #loop executes once every 0.2 seconds (= 5 Hz)

#Post ALlocation
errorCode=simxSetJointTargetVelocity(clientID,left_motor_handle,0,simx_opmode_streaming)
errorCode=simxSetJointTargetVelocity(clientID,right_motor_handle,0,simx_opmode_streaming)
